@@ -5,8 +5,6 @@
 """Data downloading and reading functions
 """
 
-from __future__ import absolute_import, print_function, division
-
 from math import log
 import os
 from os import path as op
@@ -15,11 +13,11 @@ import shutil
 import time
 
 from . import appdata_dir, resource_dirs
-from . import StdoutProgressIndicator, string_types, urlopen
+from . import StdoutProgressIndicator, urlopen
 
 
 class InternetNotAllowedError(IOError):
-    """ Plugins that need resources can just use get_remote_file(), but
+    """Plugins that need resources can just use get_remote_file(), but
     should catch this error and silently ignore it.
     """
 
@@ -27,13 +25,13 @@ class InternetNotAllowedError(IOError):
 
 
 class NeedDownloadError(IOError):
-    """ Is raised when a remote file is requested that is not locally
+    """Is raised when a remote file is requested that is not locally
     available, but which needs to be explicitly downloaded by the user.
     """
 
 
 def get_remote_file(fname, directory=None, force_download=False, auto=True):
-    """ Get a the filename for the local version of a file from the web
+    """Get a the filename for the local version of a file from the web
 
     Parameters
     ----------
@@ -82,7 +80,7 @@ def get_remote_file(fname, directory=None, force_download=False, auto=True):
                     shutil.copy(filename, filename2)
                     return filename2
                 return filename
-            if isinstance(force_download, string_types):
+            if isinstance(force_download, str):
                 ntime = time.strptime(force_download, "%Y-%m-%d")
                 ftime = time.gmtime(op.getctime(filename))
                 if ftime >= ntime:
@@ -116,7 +114,7 @@ def get_remote_file(fname, directory=None, force_download=False, auto=True):
         os.makedirs(op.abspath(op.dirname(filename)))
     # let's go get the file
     if os.getenv("CONTINUOUS_INTEGRATION", False):  # pragma: no cover
-        # On Travis, we retry a few times ...
+        # On CI, we retry a few times ...
         for i in range(2):
             try:
                 _fetch_file(url, filename)
@@ -161,7 +159,7 @@ def _fetch_file(url, file_name, print_destination=True):
     for tries in range(4):
         try:
             # Checking file size and displaying it alongside the download url
-            remote_file = urlopen(url, timeout=5.)
+            remote_file = urlopen(url, timeout=5.0)
             file_size = int(remote_file.headers["Content-Length"].strip())
             size_str = _sizeof_fmt(file_size)
             print("Try %i. Download from %s (%s)" % (tries + 1, url, size_str))
@@ -184,7 +182,7 @@ def _fetch_file(url, file_name, print_destination=True):
                     local_file.close()
     else:
         raise IOError(
-            "Unable to download %r. Perhaps there is a no internet "
+            "Unable to download %r. Perhaps there is no internet "
             "connection? If there is, please report this problem."
             % os.path.basename(file_name)
         )
@@ -231,7 +229,7 @@ def _chunk_write(chunk, local_file, progress):
     """Write a chunk to file and update the progress bar"""
     local_file.write(chunk)
     progress.increase_progress(len(chunk))
-    time.sleep(0.0001)
+    time.sleep(0)  # Give other threads a chance, e.g. those that handle stdout pipes
 
 
 def _sizeof_fmt(num):
@@ -241,7 +239,7 @@ def _sizeof_fmt(num):
     """Human friendly file size"""
     if num > 1:
         exponent = min(int(log(num, 1024)), len(units) - 1)
-        quotient = float(num) / 1024 ** exponent
+        quotient = float(num) / 1024**exponent
         unit = units[exponent]
         num_decimals = decimals[exponent]
         format_string = "{0:.%sf} {1}" % num_decimals
