@@ -6,23 +6,19 @@ Definition of the Request object, which acts as a kind of bridge between
 what the user wants and what the plugins can.
 """
 
-from __future__ import absolute_import, print_function, division
-
-import sys
 import os
 from io import BytesIO
 import zipfile
 import tempfile
 import shutil
+import enum
+import warnings
 
-from ..core import string_types, binary_type, urlopen, get_remote_file
+from ..core import urlopen, get_remote_file
 
-if sys.version_info < (3,):
-    FileNotFoundError = OSError
-try:
-    from pathlib import Path
-except ImportError:
-    Path = None
+from pathlib import Path
+from urllib.parse import urlparse
+from typing import Optional
 
 # URI types
 URI_BYTES = 1
@@ -32,6 +28,131 @@ URI_ZIPPED = 4
 URI_HTTP = 5
 URI_FTP = 6
 
+
+class IOMode(str, enum.Enum):
+    """Available Image modes
+
+    This is a helper enum for ``Request.Mode`` which is a composite of a
+    ``Request.ImageMode`` and ``Request.IOMode``. The IOMode that tells the
+    plugin if the resource should be read from or written to. Available values are
+
+    - read ("r"): Read from the specified resource
+    - write ("w"): Write to the specified resource
+
+    """
+
+    read = "r"
+    write = "w"
+
+
+class ImageMode(str, enum.Enum):
+    """Available Image modes
+
+    This is a helper enum for ``Request.Mode`` which is a composite of a
+    ``Request.ImageMode`` and ``Request.IOMode``. The image mode that tells the
+    plugin the desired (and expected) image shape. Available values are
+
+    - single_image ("i"): Return a single image extending in two spacial
+      dimensions
+    - multi_image ("I"): Return a list of images extending in two spacial
+      dimensions
+    - single_volume ("v"): Return an image extending into multiple dimensions.
+      E.g. three spacial dimensions for image stacks, or two spatial and one
+      time dimension for videos
+    - multi_volume ("V"): Return a list of images extending into multiple
+      dimensions.
+    - any_mode ("?"): Return an image in any format (the plugin decides the
+      appropriate action).
+
+    """
+
+    single_image = "i"
+    multi_image = "I"
+    single_volume = "v"
+    multi_volume = "V"
+    any_mode = "?"
+
+
+@enum.unique
+class Mode(str, enum.Enum):
+    """The mode to use when interacting with the resource
+
+    ``Request.Mode`` is a composite of ``Request.ImageMode`` and
+    ``Request.IOMode``. The image mode that tells the plugin the desired (and
+    expected) image shape and the ``Request.IOMode`` tells the plugin the way
+    the resource should be interacted with. For a detailed description of the
+    available modes, see the documentation for ``Request.ImageMode`` and
+    ``Request.IOMode`` respectively.
+
+    Available modes are all combinations of ``Request.IOMode`` and ``Request.ImageMode``:
+
+    - read_single_image ("ri")
+    - read_multi_image ("rI")
+    - read_single_volume ("rv")
+    - read_multi_volume ("rV")
+    - read_any ("r?")
+    - write_single_image ("wi")
+    - write_multi_image ("wI")
+    - write_single_volume ("wv")
+    - write_multi_volume ("wV")
+    - write_any ("w?")
+
+    Examples
+    --------
+    >>> Request.Mode("rI")  # a list of simple images should be read from the resource
+    >>> Request.Mode("wv")  # a single volume should be written to the resource
+
+    """
+
+    read_single_image = "ri"
+    read_multi_image = "rI"
+    read_single_volume = "rv"
+    read_multi_volume = "rV"
+    read_any = "r?"
+    write_single_image = "wi"
+    write_multi_image = "wI"
+    write_single_volume = "wv"
+    write_multi_volume = "wV"
+    write_any = "w?"
+
+    @classmethod
+    def _missing_(cls, value):
+        """Enable Mode("r") and Mode("w")
+
+        The sunder method ``_missing_`` is called whenever the constructor fails
+        to directly look up the corresponding enum value from the given input.
+        In our case, we use it to convert the modes "r" and "w" (from the v3
+        API) into their legacy versions "r?" and "w?".
+
+        More info on _missing_:
+        https://docs.python.org/3/library/enum.html#supported-sunder-names
+        """
+
+        if value == "r":
+            return cls("r?")
+        elif value == "w":
+            return cls("w?")
+        else:
+            raise ValueError(f"{value} is no valid Mode.")
+
+    @property
+    def io_mode(self) -> IOMode:
+        return IOMode(self.value[0])
+
+    @property
+    def image_mode(self) -> ImageMode:
+        return ImageMode(self.value[1])
+
+    def __getitem__(self, key):
+        """For backwards compatibility with the old non-enum modes"""
+        if key == 0:
+            return self.io_mode
+        elif key == 1:
+            return self.image_mode
+        else:
+            raise IndexError(f"Mode has no item {key}")
+
+
 SPECIAL_READ_URIS = "<video", "<screen>", "<clipboard>"
 
 # The user can use this string in a write call to get the data back as bytes.
@@ -40,8 +161,10 @@ RETURN_BYTES = "<bytes>"
 # Example images that will be auto-downloaded
 EXAMPLE_IMAGES = {
     "astronaut.png": "Image of the astronaut Eileen Collins",
-    "camera.png": "Classic grayscale image of a photographer",
+    "camera.png": "A grayscale image of a photographer",
     "checkerboard.png": "Black and white image of a chekerboard",
+    "wood.jpg": "A (repeatable) texture of wooden planks",
+    "bricks.jpg": "A (repeatable) texture of stone bricks",
     "clock.png": "Photo of a clock with motion blur (Stefan van der Walt)",
     "coffee.png": "Image of a cup of coffee (Rachel Michetti)",
     "chelsea.png": "Image of Stefan's cat",
@@ -58,11 +181,12 @@ EXAMPLE_IMAGES = {
     "newtonscradle.gif": "Animated GIF of a newton's cradle",
     "cockatoo.mp4": "Video file of a cockatoo",
     "stent.npz": "Volumetric image showing a stented abdominal aorta",
+    "meadow_cube.jpg": "A cubemap image of a meadow, e.g. to render a skybox.",
 }
 
 
 class Request(object):
-    """ Request(uri, mode, **kwargs)
+    """ImageResource handling utility.
 
     Represents a request for reading or saving an image resource. This
     object wraps information to that request and acts as an interface
@@ -77,7 +201,7 @@ class Request(object):
     information between different formats and between a format and
     associated reader/writer.
 
-    parameters
+    Parameters
     ----------
     uri : {str, bytes, file}
         The resource to load the image from.
@@ -86,14 +210,16 @@ class Request(object):
         request. The second character is used to indicate the kind of data:
         "i" for an image, "I" for multiple images, "v" for a volume,
         "V" for multiple volumes, "?" for don't care.
+
     """
 
-    def __init__(self, uri, mode, **kwargs):
-
+    def __init__(self, uri, mode, *, extension=None, format_hint: str = None, **kwargs):
         # General
+        self.raw_uri = uri
         self._uri_type = None
         self._filename = None
         self._extension = None
+        self._format_hint = None
         self._kwargs = kwargs
         self._result = None  # Some write actions may have a result
 
@@ -104,6 +230,7 @@ class Request(object):
 
         # To handle the plugin side
         self._file = None  # To store the file instance
+        self._file_is_local = False  # whether the data needs to be copied at end
         self._filename_local = None  # not None if using tempfile on this FS
         self._firstbytes = None  # For easy header parsing
 
@@ -111,34 +238,51 @@ class Request(object):
         # self._potential_formats = []
 
         # Check mode
-        self._mode = mode
-        if not isinstance(mode, string_types):
-            raise ValueError("Request requires mode must be a string")
-        if not len(mode) == 2:
-            raise ValueError("Request requires mode to have two chars")
-        if mode[0] not in "rw":
-            raise ValueError('Request requires mode[0] to be "r" or "w"')
-        if mode[1] not in "iIvV?":
-            raise ValueError('Request requires mode[1] to be in "iIvV?"')
+        try:
+            self._mode = Mode(mode)
+        except ValueError:
+            raise ValueError(f"Invalid Request.Mode: {mode}")
 
         # Parse what was given
         self._parse_uri(uri)
 
         # Set extension
-        if self._filename is not None:
-            ext = self._filename
-            if self._filename.startswith(("http://", "https://", "ftp://", "ftps://")):
-                ext = ext.split("?")[0]
-            self._extension = "." + ext.split(".")[-1].lower()
+        if extension is not None:
+            if extension[0] != ".":
+                raise ValueError(
+                    "`extension` should be a file extension starting with a `.`,"
+                    f" but is `{extension}`."
+                )
+            self._extension = extension
+        elif self._filename is not None:
+            if self._uri_type in (URI_FILENAME, URI_ZIPPED):
+                path = self._filename
+            else:
+                path = urlparse(self._filename).path
+            ext = Path(path).suffix.lower()
+            self._extension = ext if ext != "" else None
+
+        if format_hint is not None:
+            warnings.warn(
+                "The usage of `format_hint` is deprecated and will be removed "
+                "in ImageIO v3. Use `extension` instead.",
+                DeprecationWarning,
+            )
+
+        if format_hint is not None and format_hint[0] != ".":
+            raise ValueError(
+                "`format_hint` should be a file extension starting with a `.`,"
+                f" but is `{format_hint}`."
+            )
+
+        self.format_hint = format_hint
 
     def _parse_uri(self, uri):
-        """ Try to figure our what we were given
-        """
-        py3k = sys.version_info[0] == 3
-        is_read_request = self.mode[0] == "r"
-        is_write_request = self.mode[0] == "w"
+        """Try to figure our what we were given"""
+        is_read_request = self.mode.io_mode is IOMode.read
+        is_write_request = self.mode.io_mode is IOMode.write
 
-        if isinstance(uri, string_types):
+        if isinstance(uri, str):
             # Explicit
             if uri.startswith("imageio:"):
                 if is_write_request:
@@ -168,33 +312,19 @@ class Request(object):
             elif uri.startswith(RETURN_BYTES) and is_write_request:
                 self._uri_type = URI_BYTES
                 self._filename = uri
-            # Less explicit (particularly on py 2.x)
-            elif py3k:
+            else:
                 self._uri_type = URI_FILENAME
                 self._filename = uri
-            else:  # pragma: no cover - our ref for coverage is py3k
-                try:
-                    isfile = os.path.isfile(uri)
-                except Exception:
-                    isfile = False  # If checking does not even work ...
-                if isfile:
-                    self._uri_type = URI_FILENAME
-                    self._filename = uri
-                elif len(uri) < 256:  # Can go wrong with veeery tiny images
-                    self._uri_type = URI_FILENAME
-                    self._filename = uri
-                elif isinstance(uri, binary_type) and is_read_request:
-                    self._uri_type = URI_BYTES
-                    self._filename = "<bytes>"
-                    self._bytes = uri
-                else:
-                    self._uri_type = URI_FILENAME
-                    self._filename = uri
-        elif py3k and isinstance(uri, binary_type) and is_read_request:
+
+        elif isinstance(uri, memoryview) and is_read_request:
+            self._uri_type = URI_BYTES
+            self._filename = "<bytes>"
+            self._bytes = uri.tobytes()
+        elif isinstance(uri, bytes) and is_read_request:
             self._uri_type = URI_BYTES
             self._filename = "<bytes>"
             self._bytes = uri
-        elif Path is not None and isinstance(uri, Path):
+        elif isinstance(uri, Path):
             self._uri_type = URI_FILENAME
             self._filename = str(uri)
         # Files
@@ -202,12 +332,12 @@ class Request(object):
             if hasattr(uri, "read") and hasattr(uri, "close"):
                 self._uri_type = URI_FILE
                 self._filename = "<file>"
-                self._file = uri
+                self._file = uri  # Data must be read from here
         elif is_write_request:
             if hasattr(uri, "write") and hasattr(uri, "close"):
                 self._uri_type = URI_FILE
                 self._filename = "<file>"
-                self._file = uri
+                self._file = uri  # Data must be written here
 
         # Expand user dir
         if self._uri_type == URI_FILENAME and self._filename.startswith("~"):
@@ -215,17 +345,21 @@ class Request(object):
 
         # Check if a zipfile
         if self._uri_type == URI_FILENAME:
-            # Search for zip extension followed by a path separater
+            # Search for zip extension followed by a path separator
             for needle in [".zip/", ".zip\\"]:
                 zip_i = self._filename.lower().find(needle)
                 if zip_i > 0:
                     zip_i += 4
-                    self._uri_type = URI_ZIPPED
-                    self._filename_zip = (
-                        self._filename[:zip_i],
-                        self._filename[zip_i:].lstrip("/\\"),
-                    )
-                    break
+                    zip_path = self._filename[:zip_i]
+                    if os.path.isdir(zip_path):
+                        pass  # is an existing dir (see #548)
+                    elif is_write_request or os.path.isfile(zip_path):
+                        self._uri_type = URI_ZIPPED
+                        self._filename_zip = (
+                            zip_path,
+                            self._filename[zip_i:].lstrip("/\\"),
+                        )
+                        break
 
         # Check if we could read it
         if self._uri_type is None:
@@ -279,7 +413,10 @@ class Request(object):
 
     @property
     def filename(self):
-        """ The uri for which reading/saving was requested. This
+        """Name of the ImageResource.
+
+
+        The uri for which reading/saving was requested. This
         can be a filename, an http address, or other resource
         identifier. Do not rely on the filename to obtain the data,
         but use ``get_file()`` or ``get_local_filename()`` instead.
@@ -287,16 +424,26 @@ class Request(object):
         return self._filename
 
     @property
-    def extension(self):
-        """ The (lowercase) extension of the requested filename.
+    def extension(self) -> str:
+        """The (lowercase) extension of the requested filename.
         Suffixes in url's are stripped. Can be None if the request is
         not based on a filename.
         """
         return self._extension
 
+    @property
+    def format_hint(self) -> Optional[str]:
+        return self._format_hint
+
+    @format_hint.setter
+    def format_hint(self, format: str) -> None:
+        self._format_hint = format
+        if self._extension is None:
+            self._extension = format
+
     @property
     def mode(self):
-        """ The mode of the request. The first character is "r" or "w",
+        """The mode of the request. The first character is "r" or "w",
         indicating a read or write request. The second character is
         used to indicate the kind of data:
         "i" for an image, "I" for multiple images, "v" for a volume,
@@ -306,24 +453,23 @@ class Request(object):
 
     @property
     def kwargs(self):
-        """ The dict of keyword arguments supplied by the user.
-        """
+        """The dict of keyword arguments supplied by the user."""
         return self._kwargs
 
-    ## For obtaining data
+    # For obtaining data
 
     def get_file(self):
-        """ get_file()
+        """get_file()
         Get a file object for the resource associated with this request.
         If this is a reading request, the file is in read mode,
         otherwise in write mode. This method is not thread safe. Plugins
-        do not need to close the file when done.
+        should not close the file when done.
 
         This is the preferred way to read/write the data. But if a
         format cannot handle file-like objects, they should use
         ``get_local_filename()``.
         """
-        want_to_write = self.mode[0] == "w"
+        want_to_write = self.mode.io_mode is IOMode.write
 
         # Is there already a file?
         # Either _uri_type == URI_FILE, or we already opened the file,
@@ -333,7 +479,9 @@ class Request(object):
 
         if self._uri_type == URI_BYTES:
             if want_to_write:
+                # Create new file object, we catch the bytes in finish()
                 self._file = BytesIO()
+                self._file_is_local = True
             else:
                 self._file = BytesIO(self._bytes)
 
@@ -349,21 +497,25 @@ class Request(object):
             if want_to_write:
                 # Create new file object, we catch the bytes in finish()
                 self._file = BytesIO()
+                self._file_is_local = True
             else:
                 # Open zipfile and open new file object for specific file
                 self._zipfile = zipfile.ZipFile(filename, "r")
                 self._file = self._zipfile.open(name, "r")
-                make_file_object_support_noop_seeks(self._file)
+                self._file = SeekableFileObject(self._file)
 
         elif self._uri_type in [URI_HTTP or URI_FTP]:
             assert not want_to_write  # This should have been tested in init
-            self._file = urlopen(self.filename, timeout=5)
-            make_file_object_support_noop_seeks(self._file)
+            timeout = os.getenv("IMAGEIO_REQUEST_TIMEOUT")
+            if timeout is None or not timeout.isdigit():
+                timeout = 5
+            self._file = urlopen(self.filename, timeout=float(timeout))
+            self._file = SeekableFileObject(self._file)
 
         return self._file
 
     def get_local_filename(self):
-        """ get_local_filename()
+        """get_local_filename()
         If the filename is an existing file on this filesystem, return
         that. Otherwise a temporary file is created on the local file
         system which can be used by the format to read from or write to.
@@ -373,36 +525,46 @@ class Request(object):
             return self._filename
         else:
             # Get filename
-            if self._uri_type in (URI_HTTP, URI_FTP):
-                ext = os.path.splitext(self._filename.split("?")[0])[1]
+            if self.extension is not None:
+                ext = self.extension
             else:
                 ext = os.path.splitext(self._filename)[1]
-            self._filename_local = tempfile.mktemp(ext, "imageio_")
+            fd, self._filename_local = tempfile.mkstemp(ext, "imageio_")
+            os.close(fd)
             # Write stuff to it?
-            if self.mode[0] == "r":
+            if self.mode.io_mode == IOMode.read:
                 with open(self._filename_local, "wb") as file:
                     shutil.copyfileobj(self.get_file(), file)
             return self._filename_local
 
-    def finish(self):
-        """ finish()
-        For internal use (called when the context of the reader/writer
-        exits). Finishes this request. Close open files and process
-        results.
+    def finish(self) -> None:
+        """Wrap up this request.
+
+        Finishes any pending reads or writes, closes any open files and frees
+        any resources allocated by this request.
         """
 
-        # Init
-        bytes = None
+        if self.mode.io_mode == IOMode.write:
+            # See if we "own" the data and must put it somewhere
+            bytes = None
+            if self._filename_local:
+                bytes = Path(self._filename_local).read_bytes()
+            elif self._file_is_local:
+                self._file_is_local = False
+                bytes = self._file.getvalue()
 
-        # Collect bytes from temp file
-        if self.mode[0] == "w" and self._filename_local:
-            with open(self._filename_local, "rb") as file:
-                bytes = file.read()
-
-        # Collect bytes from BytesIO file object.
-        written = (self.mode[0] == "w") and self._file
-        if written and self._uri_type in [URI_BYTES, URI_ZIPPED]:
-            bytes = self._file.getvalue()
+            # Put the data in the right place
+            if bytes is not None:
+                if self._uri_type == URI_BYTES:
+                    self._result = bytes  # Picked up by imread function
+                elif self._uri_type == URI_FILE:
+                    self._file.write(bytes)
+                elif self._uri_type == URI_ZIPPED:
+                    zf = zipfile.ZipFile(self._filename_zip[0], "a")
+                    zf.writestr(self._filename_zip[1], bytes)
+                    zf.close()
+                # elif self._uri_type == URI_FILENAME: -> is always direct
+                # elif self._uri_type == URI_FTP/HTTP: -> write not supported
 
         # Close open files that we know of (and are responsible for)
         if self._file and self._uri_type != URI_FILE:
@@ -411,36 +573,32 @@ class Request(object):
         if self._zipfile:
             self._zipfile.close()
             self._zipfile = None
+
         # Remove temp file
         if self._filename_local:
             try:
                 os.remove(self._filename_local)
             except Exception:  # pragma: no cover
-                pass
+                warnings.warn(
+                    "Failed to delete the temporary file at "
+                    f"`{self._filename_local}`. Please report this issue."
+                )
             self._filename_local = None
 
-        # Handle bytes that we collected
-        if bytes is not None:
-            if self._uri_type == URI_BYTES:
-                self._result = bytes  # Picked up by imread function
-            elif self._uri_type == URI_ZIPPED:
-                zf = zipfile.ZipFile(self._filename_zip[0], "a")
-                zf.writestr(self._filename_zip[1], bytes)
-                zf.close()
-
         # Detach so gc can clean even if a reference of self lingers
         self._bytes = None
 
     def get_result(self):
-        """ For internal use. In some situations a write action can have
+        """For internal use. In some situations a write action can have
         a result (bytes data). That is obtained with this function.
         """
+        # Is there a reason to disallow reading multiple times?
         self._result, res = None, self._result
         return res
 
     @property
     def firstbytes(self):
-        """ The first 256 bytes of the file. These can be used to
+        """The first 256 bytes of the file. These can be used to
         parse the header to determine the file-format.
         """
         if self._firstbytes is None:
@@ -456,7 +614,7 @@ class Request(object):
                 f = self.get_file()
             except IOError:
                 if os.path.isdir(self.filename):  # A directory, e.g. for DICOM
-                    self._firstbytes = binary_type()
+                    self._firstbytes = bytes()
                     return
                 raise
             try:
@@ -479,12 +637,12 @@ class Request(object):
 
 
 def read_n_bytes(f, N):
-    """ read_n_bytes(file, n)
+    """read_n_bytes(file, n)
 
     Read n bytes from the given file, or less if the file has less
     bytes. Returns zero bytes if the file is closed.
     """
-    bb = binary_type()
+    bb = bytes()
     while len(bb) < N:
         extra_bytes = f.read(N - len(bb))
         if not extra_bytes:
@@ -493,33 +651,101 @@ def read_n_bytes(f, N):
     return bb
 
 
-def make_file_object_support_noop_seeks(f):
-    """This fixes up an HTTPResponse object so that it can tell(), and also
-    seek() will work if its effectively a no-op. This allows tools like Pillow
-    to use the file object.
+class SeekableFileObject:
+    """A readonly wrapper file object that add support for seeking, even if
+    the wrapped file object does not. The allows us to stream from http and
+    still use Pillow.
     """
-    count = [0]
 
-    def read(n=None):
-        res = ori_read(n)
-        count[0] += len(res)
+    def __init__(self, f):
+        self.f = f
+        self._i = 0  # >=0 but can exceed buffer
+        self._buffer = b""
+        self._have_all = False
+        self.closed = False
+
+    def read(self, n=None):
+        # Fix up n
+        if n is None:
+            pass
+        else:
+            n = int(n)
+            if n < 0:
+                n = None
+
+        # Can and must we read more?
+        if not self._have_all:
+            more = b""
+            if n is None:
+                more = self.f.read()
+                self._have_all = True
+            else:
+                want_i = self._i + n
+                want_more = want_i - len(self._buffer)
+                if want_more > 0:
+                    more = self.f.read(want_more)
+                    if len(more) < want_more:
+                        self._have_all = True
+            self._buffer += more
+
+        # Read data from buffer and update pointer
+        if n is None:
+            res = self._buffer[self._i :]
+        else:
+            res = self._buffer[self._i : self._i + n]
+        self._i += len(res)
+
         return res
 
-    def tell():
-        return count[0]
+    def tell(self):
+        return self._i
 
-    def seek(i, mode=0):
-        if not (mode == 0 and i == count[0]):
-            ori_seek(i, mode)
+    def seek(self, i, mode=0):
+        # Mimic BytesIO behavior
 
-    def fail_seek(i, mode=0):
-        raise RuntimeError("No seeking allowed!")
+        # Get the absolute new position
+        i = int(i)
+        if mode == 0:
+            if i < 0:
+                raise ValueError("negative seek value " + str(i))
+            real_i = i
+        elif mode == 1:
+            real_i = max(0, self._i + i)  # negative ok here
+        elif mode == 2:
+            if not self._have_all:
+                self.read()
+            real_i = max(0, len(self._buffer) + i)
+        else:
+            raise ValueError("invalid whence (%s, should be 0, 1 or 2)" % i)
 
-    # Note, there is currently no protection from wrapping an object more than
-    # once, it will (probably) work though, because closures.
-    ori_read = f.read
-    ori_seek = f.seek if hasattr(f, "seek") else fail_seek
+        # Read some?
+        if real_i <= len(self._buffer):
+            pass  # no need to read
+        elif not self._have_all:
+            assert real_i > self._i  # if we don't have all, _i cannot be > _buffer
+            self.read(real_i - self._i)  # sets self._i
 
-    f.read = read
-    f.tell = tell
-    f.seek = seek
+        self._i = real_i
+        return self._i
+
+    def close(self):
+        self.closed = True
+        self.f.close()
+
+    def isatty(self):
+        return False
+
+    def seekable(self):
+        return True
+
+
+class InitializationError(Exception):
+    """The plugin could not initialize from the given request.
+
+    This is a _internal_ error that is raised by plugins that fail to handle
+    a given request. We use this to differentiate incompatibility between
+    a plugin and a request from an actual error/bug inside a plugin.
+
+    """
+
+    pass
